package java.security.cert;

import java.io.IOException;
import sun.security.x509.NameConstraintsExtension;
import javax.security.auth.x500.X500Principal;
import java.security.PublicKey;

public class TrustAnchor
{
    private final PublicKey pubKey;
    private final String caName;
    private final X500Principal caPrincipal;
    private final X509Certificate trustedCert;
    private byte[] ncBytes;
    private NameConstraintsExtension nc;
    
    public TrustAnchor(X509Certificate trustedCert, byte[] nameConstraints) {
        if (trustedCert == null) {
            throw new NullPointerException("the trustedCert parameter must be non-null");
        }
        this.trustedCert = trustedCert;
        this.pubKey = null;
        this.caName = null;
        this.caPrincipal = null;
        this.setNameConstraints(nameConstraints);
        return;
    }
    
    public TrustAnchor(X500Principal caPrincipal, PublicKey pubKey, byte[] nameConstraints) {
        if (caPrincipal == null || pubKey == null) {
            throw new NullPointerException();
        }
        this.trustedCert = null;
        this.caPrincipal = caPrincipal;
        this.caName = caPrincipal.getName();
        this.pubKey = pubKey;
        this.setNameConstraints(nameConstraints);
        return;
    }
    
    public TrustAnchor(String s, PublicKey pubKey, byte[] nameConstraints) {
        if (pubKey == null) {
            throw new NullPointerException("the pubKey parameter must be non-null");
        }
        if (s == null) {
            throw new NullPointerException("the caName parameter must be non-null");
        }
        if (s.length() == 0) {
            throw new IllegalArgumentException("the caName parameter must be a non-empty String");
        }
        this.caPrincipal = new X500Principal(s);
        this.pubKey = pubKey;
        this.caName = s;
        this.trustedCert = null;
        this.setNameConstraints(nameConstraints);
        return;
    }
    
    public final X509Certificate getTrustedCert() {
        return this.trustedCert;
    }
    
    public final X500Principal getCA() {
        return this.caPrincipal;
    }
    
    public final String getCAName() {
        return this.caName;
    }
    
    public final PublicKey getCAPublicKey() {
        return this.pubKey;
    }
    
    private void setNameConstraints(byte[] value) {
        if (value == null) {
            this.ncBytes = null;
            this.nc = null;
        }
        else {
            this.ncBytes = (byte[])value.clone();
            try {
                this.nc = new NameConstraintsExtension(Boolean.FALSE, value);
            }
            catch (IOException cause) {
                IllegalArgumentException ex = new IllegalArgumentException(cause.getMessage());
                ex.initCause(cause);
                throw ex;
            }
        }
    }
    
    public final byte[] getNameConstraints() {
        return (this.ncBytes == null) ? null : ((byte[])this.ncBytes.clone());
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[\n");
        if (this.pubKey != null) {
            sb.append("  Trusted CA Public Key: " + this.pubKey.toString() + "\n");
            sb.append("  Trusted CA Issuer Name: " + this.caName + "\n");
        }
        else {
            sb.append("  Trusted CA cert: " + this.trustedCert.toString() + "\n");
        }
        if (this.nc != null) {
            sb.append("  Name Constraints: " + this.nc.toString() + "\n");
        }
        return sb.toString();
    }
}
